<?php

// Author: de77
// Licence: MIT
// First-version: 9.02.2010
// Version: 26.01.2018
// http://de77.com
// Works with TGA 8bpp grayscale, 24bpp and 32bpp; both normal and upside-down
// Does not work with color-mapped images (type 1 and 9)

function getimagesizetga($filename)
{
	$f = fopen($filename, 'rb');
	$header = fread($f, 18);
	$header = @unpack(	"cimage_id_len/ccolor_map_type/cimage_type/vcolor_map_origin/vcolor_map_len/" .
						"ccolor_map_entry_size/vx_origin/vy_origin/vwidth/vheight/" .
						"cpixel_size/cdescriptor", $header);
	fclose($f);
	
	$types = array(0,1,2,3,9,10,11,32,33);		
	if (!in_array($header['image_type'], $types))
	{
		return array(0, 0, 0, 0, 0);
	}
		
	if ($header['pixel_size'] > 32)
	{
		return array(0, 0, 0, 0, 0);
	}
		
	return array($header['width'], $header['height'], 'tga', $header['pixel_size'], $header['image_type']);
}

function imagetga($im, $filename)
{
	list($width, $height) = array(imagesx($im), imagesy($im));
	
	$tga = "\0\0\2\0\0\0\0\0\0\0\0\0" . pack('vv', $width, $height) . '  ';
	
	for ($y=0; $y<$height; $y++)
	for ($x=0; $x<$width; $x++)
	{
		$rgb = imagecolorat($im, $x, $y);
		$r = ($rgb >> 16) & 0xFF;
		$g = ($rgb >> 8) & 0xFF;
		$b = $rgb & 0xFF;
		$a = ($rgb >> 24) & 0x7F;
		$tga .= chr($b).chr($g).chr($r).chr((127-$a)*2);
	}
	 
	file_put_contents($filename, $tga);
}

function imagecreatefromtga($filename)
{
	$f = fopen($filename, 'rb');
	if (!$f)
	{
		return false;
	}
	$header = fread($f, 18);
	$header = unpack(	"cimage_id_len/ccolor_map_type/cimage_type/vcolor_map_origin/vcolor_map_len/" .
						"ccolor_map_entry_size/vx_origin/vy_origin/vwidth/vheight/" .
						"cpixel_size/cdescriptor", $header);

	switch ($header['image_type'])
	{		
		case 2:		//truecolor, uncompressed
		case 3:		//grayscale, uncompressed		
		case 10:	//truecolor, rle
		case 11:	//grayscale, rle		
					break;
		default:	die('Unsupported TGA format');					
	}

	if ($header['pixel_size'] != 8 and $header['pixel_size'] != 24 and $header['pixel_size'] != 32)
	{
		die('Unsupported TGA color depth');	
	}
	
	$bytes = $header['pixel_size'] / 8;
	
	if ($header['image_id_len'] > 0)
	{
		$header['image_id'] = fread($f, $header['image_id_len']);
	}
	else
	{
		$header['image_id'] = '';	
	}
	
	$im = imagecreatetruecolor($header['width'], $header['height']);
	imagealphablending($im, 0);
	
	$size = $header['width'] * $header['height'] * $bytes;
	 
	//-- check whether this is NEW TGA or not
	$pos = ftell($f);
	fseek($f, -26, SEEK_END);	
	$newtga = fread($f, 26);
	if (substr($newtga, 8, 16) != 'TRUEVISION-XFILE')
	{
		$newtga = false;
	}
	
	fseek($f, 0, SEEK_END);
	$datasize = ftell($f) - $pos; 
	if ($newtga)
	{
		$datasize -= 26;
	}
	
	fseek($f, $pos, SEEK_SET);

	//-- end of check
	$data = fread($f, $datasize);   
	if ($header['image_type'] == 10 or $header['image_type'] == 11)
	{
		$data = rle_decode($data, $size, $bytes);					
	}
	if (bit5($header['descriptor']) == 1)
	{
		$reverse = true;	
	}
	else
	{
		$reverse = false;
	}    
    
	$i = 0;
	
	//read pixels
	if ($reverse)
	{   
	    for ($y=0; $y<$header['height']; $y++)	
    	for ($x=0; $x<$header['width']; $x++)
    	{
    		if ($bytes == 1)
    		{
    			$col = ord($data[$i]) + (ord($data[$i])<<8) + (ord($data[$i])<<16);
    		}
    		else
    		{
				$col = ord($data[$i]) + (ord($data[$i+1])<<8) + (ord($data[$i+2])<<16);
				if ($bytes == 4)
				{
					$col += ((255-ord($data[$i+3]))<<24);
				}
			}
			imagesetpixel($im, $x, $y, $col);
    		$i += $bytes;
    	}
    }
    else
    {
        for ($y=$header['height']-1; $y>=0; $y--)		
    	for ($x=0; $x<$header['width']; $x++)
    	{
    		if ($bytes == 1)
    		{
    			$col = ord($data[$i]) + (ord($data[$i])<<8) + (ord($data[$i])<<16);
    		}
    		else
    		{
				$col = ord($data[$i]) + (ord($data[$i+1])<<8) + (ord($data[$i+2])<<16);
				if ($bytes == 4)
				{
					$col += ((255-ord($data[$i+3]))<<24);
				}
			}
			imagesetpixel($im, $x, $y, $col);
    		$i += $bytes;   		
    	}
    }
	fclose($f);    	    
	
	return $im;
}

function bit5($x)
{
	return ($x & 32) >> 5;	
}

function rle_decode($data, $datalen, $bytes)
{
	$len = strlen($data);	
	$out = '';
	
	$i = 0;
	$k = 0;
	while ($i<$len)
	{
		dec_bits(ord($data[$i]), $type, $value);
		if ($k >= $datalen)
		{
			break;
		}

		$i++;
		
		if ($type == 0) //raw
		{
			for ($j=0; $j<$bytes*$value; $j++)
			{
				$out .= $data[$j+$i];
				$k++;			
			}
			$i += $value*$bytes;
		}
		else //rle
		{
			for ($j=0; $j<$value; $j++)
			{
				$out .= substr($data, $i, $bytes);
				$k++;				
			}			
			$i += $bytes;
		}	
	}
	return $out;
}

function dec_bits($byte, &$type, &$value)
{
	$type = ($byte & 0x80) >> 7;
	$value = 1 + ($byte & 0x7F);
}